# Configuration file for the Sphinx documentation builder.

import datetime
import os
import sys

# Get configuration information from setup.cfg
try:
    from ConfigParser import ConfigParser
except ImportError:
    from configparser import ConfigParser
conf = ConfigParser()

conf.read([os.path.join(os.path.dirname(__file__), '..', 'setup.cfg')])
setup_cfg = dict(conf.items('metadata'))

# By default, highlight as Python 3.
highlight_language = 'python3'

# exclude_patterns.append('_templates')

# -- Project information

project = setup_cfg['name']
author = setup_cfg['author']
copyright = '{0}, {1}'.format(
    datetime.datetime.now().year, setup_cfg['author'])

__import__(project)
package = sys.modules[project]

ver = package.__version__
version = '.'.join(ver.split('.'))[:5]
release = ver

html_title = '{0} v{1}'.format(project, release)

# The master toctree document.
root_doc = 'index'

man_pages = [('index', project.lower(), project + u' Documentation',
              [author], 1)]

# -- General configuration
extensions = [
    'sphinx.ext.duration',
    'sphinx.ext.doctest',
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.intersphinx',
]

intersphinx_mapping = {
    'python': ('https://docs.python.org/3/', None),
    'sphinx': ('https://www.sphinx-doc.org/en/master/', None),
}
intersphinx_disabled_domains = ['std']

templates_path = ['_templates']

# -- Options for HTML output
html_theme = "classic"

# -- Options for EPUB output
# epub_show_urls = 'footnote'
# github_issues_url = 'https://github.com/astropy/ccdproc/issues/'
# nitpicky = True






"""
Author: Kyle Koeller
Date Created: 03/08/2023

Last Edited: 04/05/2023
This program queries Gaia DR3, to gather specific parameters
https://gea.esac.esa.int/archive/
https://iopscience.iop.org/article/10.3847/1538-3881/acaaa7/pdf
https://iopscience.iop.org/article/10.3847/1538-3881/ab3467/pdf
https://arxiv.org/pdf/2012.01916.pdf
"""

from pyia import GaiaData
import pandas as pd
from .vseq_updated import splitter
# from vseq_updated import splitter  # testing
import math as mt


def target_star():
    """
    This queries Gaia DR3 all the parameters below, but I only outputted the specific parameters that are (at the moment)
    the most important for current research at BSU

    :return: Outputs a file with the specific parameters
    """
    # 13:27:50.4728234064 75:39:45.384765984
    # 00:28:27.9684836736 78:57:42.657327180
    ra_input = input("\nEnter the RA of your system (HH:MM:SS.SSSS): ")
    dec_input = input("Enter the DEC of your system (DD:MM:SS.SSSS or -DD:MM:SS.SSSS): ")

    ra_input2 = splitter([ra_input])
    dec_input2 = splitter([dec_input])

    ra = ra_input2[0] * 15
    dec = dec_input2[0]

    # tess_mag(ra, dec)

    g = GaiaData.from_query("""
    SELECT TOP 2000 
    gaia_source.source_id,gaia_source.ra,gaia_source.dec,gaia_source.parallax,gaia_source.parallax_error,
    gaia_source.pmra,gaia_source.pmdec,gaia_source.ruwe,gaia_source.phot_g_mean_mag,gaia_source.bp_rp,
    gaia_source.radial_velocity,gaia_source.radial_velocity_error,gaia_source.rv_method_used,
    gaia_source.phot_variable_flag,gaia_source.non_single_star,gaia_source.has_xp_continuous,
    gaia_source.has_xp_sampled,gaia_source.has_rvs,gaia_source.has_epoch_photometry,gaia_source.has_epoch_rv,
    gaia_source.has_mcmc_gspphot,gaia_source.has_mcmc_msc,gaia_source.teff_gspphot,gaia_source.teff_gspphot_lower,
    gaia_source.teff_gspphot_upper,gaia_source.logg_gspphot,
    gaia_source.mh_gspphot,gaia_source.distance_gspphot,gaia_source.distance_gspphot_lower,
    gaia_source.distance_gspphot_upper,gaia_source.azero_gspphot,gaia_source.ag_gspphot,
    gaia_source.ebpminrp_gspphot,gaia_source.phot_g_mean_mag,gaia_source.phot_bp_mean_mag,gaia_source.phot_rp_mean_mag
    FROM gaiadr3.gaia_source 
    WHERE 
    CONTAINS(
        POINT('ICRS',gaiadr3.gaia_source.ra,gaiadr3.gaia_source.dec),
        CIRCLE(
            'ICRS',
            COORD1(EPOCH_PROP_POS({}, {},4.7516,48.8840,-24.1470,0,2000,2016.0)),
            COORD2(EPOCH_PROP_POS({}, {},4.7516,48.8840,-24.1470,0,2000,2016.0)),
            0.001388888888888889)
    )=1""".format(ra, dec, ra, dec))

    # to add parameters to the output file, add them here and the format for the parameter is 'g.[param name from above]'
    df = pd.DataFrame({
        "Parallax(mas)": g.parallax[:4],
        "Parallax_err(mas)": g.parallax_error[:4],
        "Distance_lower(pc)": g.distance_gspphot_lower[:4],
        "Distance(pc)": g.distance_gspphot[:4],
        "Distance_higher(pc)": g.distance_gspphot[:4],
        "T_eff_lower(K)": g.teff_gspphot_lower[:4],
        "T_eff(K)": g.teff_gspphot[:4],
        "T_eff_higher(K)": g.teff_gspphot_upper[:4],
        "G_Mag": g.phot_g_mean_mag[:4],
        "G_BP_Mag": g.phot_bp_mean_mag[:4],
        "G_RP_Mag": g.phot_rp_mean_mag[:4],
        "Radial_velocity(km/s)": g.radial_velocity[:4],
        "Radial_velocity_err(km/s)": g.radial_velocity_error[:4],
    })

    text_file = input("\nEnter a text file pathway and name for Gaia data "
                      "(ex: C:\\folder1\\Gaia_254037.txt): ")
    df.to_csv(text_file, index=None, sep="\t")

    print("\n For more information on each of the output parameters please reference this webpage: "
          "https://gea.esac.esa.int/archive/documentation/GDR3/Gaia_archive/chap_datamodel/sec_dm_main_source_catalogue/ssec_dm_gaia_source.html")
    print("If any of the parameters have values of '1e+20', then Gaia does not have data on that specific parameter.")

    print("\nCompleted save.\n")


def tess_mag(ra, dec):
    """
    Calculates TESS magnitudes for comparison stars

    :param ra: List of RA's for comparison stars
    :param dec: List of DEC's for comparison stars

    :return: list of TESS magnitudes and errors
    """
    T_list = []
    T_err_list = []
    for count, val in enumerate(ra):
        # the query searching for the magnitudes and fluxes
        g = GaiaData.from_query("""
            SELECT TOP 2000 gaia_source.source_id,gaia_source.ra,gaia_source.dec,
            gaia_source.phot_g_mean_flux_over_error,gaia_source.phot_g_mean_mag,
            gaia_source.phot_bp_mean_flux_over_error,gaia_source.phot_bp_mean_mag,
            gaia_source.phot_rp_mean_flux_over_error,gaia_source.phot_rp_mean_mag
            FROM gaiadr3.gaia_source 
            WHERE 
            CONTAINS(
                POINT('ICRS',gaiadr3.gaia_source.ra,gaiadr3.gaia_source.dec),
                CIRCLE('ICRS',{},{}, {})
        )=1""".format(val*15, dec[count], 0.0008333333333333334))
        # 0.0002777777777777778 is 1 arcsecond,
        # 0.0005555555555555556 is 2 arcseconds,
        # 0.0008333333333333334 is 3 arcseconds
        # 0.001388888888888889 is 5 arcseconds

        # 13:27:50.4728234064 75:39:45.384765984
        # 00:28:27.9684836736 78:57:42.657327180

        """
        Each mag and flux from Gaia's filters
        Use .value at the end of each Gaia variable to get only the number and not the unit
        The fluxes are coming in as arrays and not quantities, so the .value cannot be applied like for the magnitudes
        """
        G = g.phot_g_mean_mag[:4].value
        G_flux = g.phot_g_mean_flux_over_error[:4]
        BP = g.phot_bp_mean_mag[:4].value
        BP_flux = g.phot_bp_mean_flux_over_error[:4]
        RP = g.phot_rp_mean_mag[:4].value
        RP_flux = g.phot_rp_mean_flux_over_error[:4]

        if len(BP) == 0 or len(RP) == 0:
            # this equation is used if there are no BP or RP magnitudes
            if len(G) == 0 or len(G_flux) == 0:
                T_list.append(99.999)
                T_err_list.append(99.999)
            else:
                T = G - 0.403
                G_err = (2.5 / mt.log(10)) * (G_flux[0] ** -1)
                T_list.append(T[0])
                T_err_list.append(G_err)
        else:
            # calculate the TESS magnitude (T) and corresponding error (T_err)
            T = G - 0.00522555 * (BP - RP) ** 3 + 0.0891337 * (BP - RP) ** 2 - 0.633923 * (BP - RP) + 0.0324473
            T = T[0]

            # error propagation formulae
            G_err = (2.5 / mt.log(10)) * (G_flux[0] ** -1)
            BP_err = (2.5 / mt.log(10)) * (BP_flux[0] ** -1)
            RP_err = (2.5 / mt.log(10)) * (RP_flux[0] ** -1)

            Bp_Rp_err = mt.sqrt(BP_err ** 2 + RP_err ** 2)
            T_err = mt.sqrt(G_err ** 2 + (Bp_Rp_err * 3) ** 2)

            # append to respective lists with only 2 decimal places, instead of ~15
            T_list.append(float(format(T, ".2f")))
            T_err_list.append(float(format(T_err, ".3f")))

    return T_list, T_err_list


if __name__ == '__main__':
    target_star()



"""
Analyze images using aperture photometry within Python and not with Astro ImageJ (AIJ)

Author: Kyle Koeller
Created: 05/07/2023
Last Updated: 08/24/2023
"""

# Python imports
import numpy as np
import pandas as pd
from pathlib import Path
import matplotlib.pyplot as plt
# import time
import warnings
from tqdm import tqdm

# Astropy imports
import ccdproc as ccdp
from astropy.coordinates import SkyCoord
from astropy.io import fits
# from astropy.nddata import CCDData
# from astropy.stats import sigma_clipped_stats
from photutils.aperture import CircularAperture, CircularAnnulus, aperture_photometry, ApertureStats
from astropy.wcs import WCS
import astropy.units as u
from astropy import wcs

# from astropy.visualization import ZScaleInterval

# turn off this warning that just tells the user,
# "The warning raised when the contents of the FITS header have been modified to be standards compliant."
warnings.filterwarnings("ignore", category=wcs.FITSFixedWarning)


def main(path="", pipeline=False, radec_list=None, obj_name=""):
    """
    Main function for aperture photometry

    Parameters
    ----------
    radec_list: list
        RADEC files for each filter
    obj_name: str
        Name of the target
    path : str
        Path to the folder containing the images.
    pipeline : bool
        If True, then the program is being run from the pipeline and will not ask for user input.
    Returns
    -------
    N/A
    """
    filt_list = ["Empty/B", "Empty/V", "Empty/R"]
    if not pipeline:
        # path = "D:\Research\Data\\NSVS_254037\\2018.09.18-reduced"  # For testing purposes
        path = input(
            "Please enter a file pathway (i.e. C:\\folder1\\folder2\\[raw]) to where the reduced images are or type "
            "the word 'Close' to leave: ")
        # allows the user to input where the raw images are and where the calibrated images go to
        radec_file = ""
        while True:
            try:
                images_path = Path(path)
                break
            except FileNotFoundError:
                print("File not found. Please try again.")
                path = input(
                    "Please enter a file pathway (i.e. C:\\folder1\\folder2\\[reduced]) to where the reduced images are or type "
                    "the word 'Close' to leave: ")

        if path.lower() == "close":
            exit()

        science_imagetyp = 'LIGHT'
        files = ccdp.ImageFileCollection(images_path)

        for filt in filt_list:
            if "/B" in filt:
                radec_file = input("Enter the file location for the RADEC file for the B filter: ")
            elif "/V" in filt:
                radec_file = input("Enter the file location for the RADEC file for the V filter: ")
            elif "/R" in filt:
                radec_file = input("Enter the file location for the RADEC file for the R filter: ")

            image_list = files.files_filtered(imagetyp=science_imagetyp, filter=filt)
            multiple_AP(image_list, images_path, filt, pipeline=pipeline, radec_file=radec_file)
    else:
        images_path = Path(path)

        science_imagetyp = 'LIGHT'

        files = ccdp.ImageFileCollection(images_path)

        for filt in filt_list:
            if "/B" in filt:
                radec_file = radec_list[0]
            elif "/V" in filt:
                radec_file = radec_list[1]
            elif "/R" in filt:
                radec_file = radec_list[2]
            image_list = files.files_filtered(imagetyp=science_imagetyp, filter=filt)
            substring_to_match = obj_name
            filtered_image_list = [file for file in image_list if substring_to_match in file]
            multiple_AP(filtered_image_list, images_path, filt, pipeline=pipeline, radec_file=radec_file)


def multiple_AP(image_list, path, filter, pipeline=False, radec_file=""):
    """
    Perform multi-aperture photometry on a list of images for a single target

    Parameters
    ----------
    filter: String
        Filter used for the images
    pipeline: Boolean
        If True, then the program is being run from the pipeline and will not ask for user input.
    radec_file: string
        Location of a radec file. If not given, the user will be prompted to enter one.
    path : pathway
        Path to the folder containing the images.
    image_list : List
        Images to perform multi-aperture photometry on.

    Returns
    -------
    None
    """

    # Define the aperture parameters
    # Define the aperture and annulus radii
    aperture_radius = 20
    annulus_radii = (30, 50)

    read_noise = 10.83  # * u.electron  # gathered from fits headers manually
    # gain = 1.43  # * u.electron / u.adu  # gathered from fits headers manually
    # F_dark = 0.01  # dark current in u.electron / u.pix / u.s

    if not pipeline:
        while True:
            try:
                # df = pd.read_csv('NSVS_254037-B.radec', skiprows=7, sep=",", header=None)
                df = pd.read_csv(radec_file, skiprows=7, sep=",", header=None)
                break
            except FileNotFoundError:
                print("File not found. Please try again.")
                radec_file = input("Please enter the RADEC file (i.e. C://folder1//folder2//[file name]: ")
    else:
        df = pd.read_csv(radec_file, skiprows=7, sep=",", header=None)
        print("RADEC file found.\n")

    magnitudes_comp = df[4]

    magnitudes_comp = magnitudes_comp.replace(99.999, pd.NA).dropna().reset_index(drop=True)

    ra = df[0]
    dec = df[1]
    # ref_star = df[2]
    # centroid = df[3]  # Not used (I don't think at least)

    magnitudes = []
    mag_err = []
    hjd = []
    bjd = []

    for icount, image_file in tqdm(enumerate(image_list), desc="Performing aperture photometry on {} images".format(len(image_list))):
        image_data, header = fits.getdata(path / image_file, header=True)
        # All the following up till the 'if' statement stays under the for loop due to needing the header information
        wcs_ = WCS(header)

        # ccd = CCDData(image_data, wcs=wcs, unit='adu')

        # Convert RA and DEC to pixel positions
        sky_coords = SkyCoord(ra, dec, unit=(u.h, u.deg), frame='icrs')
        pixel_coords = wcs_.world_to_pixel(sky_coords)

        x_coords, y_coords = pixel_coords

        # target_position = np.array(pixel_coords[0])
        # comparison_positions = np.array(pixel_coords[1:])
        target_position = (x_coords[0], y_coords[0])
        comparison_positions = list(zip(x_coords[1:], y_coords[1:]))

        hjd.append(header['HJD-OBS'])
        bjd.append(header['BJD-OBS'])

        # Create the apertures and annuli
        target_aperture = CircularAperture(target_position, r=aperture_radius)
        target_annulus = CircularAnnulus(target_position, *annulus_radii)

        comparison_aperture = [CircularAperture(pos1, r=aperture_radius) for pos1 in comparison_positions]
        comparison_annulus = [CircularAnnulus(pos2, *annulus_radii) for pos2 in comparison_positions]
        target_phot_table = aperture_photometry(image_data, target_aperture)
        # comparison_phot_table = aperture_photometry(image_data, comparison_aperture)

        if icount == 0:
            im_plot(image_data, target_aperture, comparison_aperture, target_annulus, comparison_annulus)
            # Create a figure and axis
            _, ax = plt.subplots(figsize=(11, 8))

        comparison_phot_table = []
        for comp_aperture, comp_annulus in zip(comparison_aperture, comparison_annulus):
            # Perform aperture photometry on the star
            aperture_phot_table = aperture_photometry(image_data, comp_aperture)

            # Perform aperture photometry on the annulus (background)
            annulus_phot_table = aperture_photometry(image_data, comp_annulus)

            # Store the result in the comparison_phot_table list
            comparison_phot_table.append((aperture_phot_table, annulus_phot_table))

        # Perform annulus photometry to estimate the background
        target_bkg_mean = ApertureStats(image_data, target_annulus).mean
        # comparison_bkg_mean = ApertureStats(image_data, comparison_annulus).mean

        # Calculate the total background for the comparison stars
        comparison_bkg_mean = []
        for annulus in comparison_annulus:
            stats = ApertureStats(image_data, annulus)
            if np.isnan(stats.mean) or np.isinf(stats.mean):
                comparison_bkg_mean.append(0)
            else:
                comparison_bkg_mean.append(stats.mean)

        # Calculate the total background for the target star
        if np.isnan(target_bkg_mean) or np.isinf(target_bkg_mean):
            target_bkg_mean = 0

        # Multiply the background mean by the aperture area to get the total background
        target_bkg = target_bkg_mean * target_aperture.area
        comparison_bkg = [bkg_mean * aperture.area for bkg_mean, aperture in
                          zip(comparison_bkg_mean, comparison_aperture)]

        # target_bkg = ApertureStats(image_data, target_aperture, local_bkg=target_bkg_mean).sum
        # # comparison_bkg = ApertureStats(image_data, comparison_aperture, local_bkg=comparison_bkg_mean).sum
        #
        # comparison_bkg = []
        # for aperture, bkg_mean in zip(comparison_aperture, comparison_bkg_mean):
        #     stats = ApertureStats(image_data, aperture, local_bkg=bkg_mean)
        #     comparison_bkg.append(stats.sum)

        # Calculate the background subtracted counts
        target_flx = target_phot_table['aperture_sum'] - target_bkg
        target_flux_err = np.sqrt(target_phot_table['aperture_sum'] + target_aperture.area * read_noise**2)
        # comparison_flx = comparison_phot_table['aperture_sum'] - comparison_bkg
        # comp_flux_err = np.sqrt(comparison_phot_table['aperture_sum'] + comparison_aperture.area * read_noise ** 2)
        comparison_flx = [phot_table[0]['aperture_sum'] - bkg
                          for phot_table, bkg in zip(comparison_phot_table, comparison_bkg)]

        comp_flux_err = [np.sqrt(phot_table[0]['aperture_sum'] + aperture.area * read_noise ** 2)
                         for phot_table, aperture in zip(comparison_phot_table, comparison_aperture)]
        comp_flux_err = np.array(comp_flux_err)

        # calculate the relative flux for each comparison star and the target star
        # rel_flx_T1 = target_flx / sum(comparison_flx)
        count = 0
        rel_flux_comps = []
        for i in comparison_flx:
            if i == comparison_flx[count]:
                rel_flux_c = i / (sum(comparison_flx) - i)
                rel_flux_comps.append(rel_flux_c)
            count += 1

        # rel_flux_comps = np.array(rel_flux_comps)

        # find the number of pixels used to estimate the sky background
        # n_b = (np.pi * annulus_radii[1]**2) - (np.pi * annulus_radii[0] ** 2)  # main equation
        # n_b_mask_comp = comparison_annulus.to_mask(method="center")
        # n_b_comp = np.sum(n_b_mask_comp)

        # n_b_mask_tar = target_annulus.to_mask(method="center")
        # n_b_tar = np.sum(n_b_mask_tar.data)

        """
        # find the number of pixels used in the aperture if the radius of the apertures is in arcseconds not pixels
        focal_length = 4114  # mm
        pixel_size = 9  # microns
        pixel_size = pixel_size * 10 ** -3  # mm
        ap_area = np.pi * aperture_radius.area**2  # area of the aperture in mm^2
        plate_scale = 1/focal_length  # rad/mm
        plate_scale = plate_scale * 206265  # arcsec/mm
        n_pix = ap_area / (plate_scale * pixel_size)**2  # number of pixels in the aperture
        """
        """
        n_pix = np.pi * aperture_radius**2  # number of pixels in the aperture

        # Calculate the total noise
        sigma_f = 0.289  # quoted from Collins 2017 https://iopscience.iop.org/article/10.3847/1538-3881/153/2/77/pdf
        F_s = 0.01  # number of sky background counts per pixel in ADU

        # N_comp = np.sqrt(gain * comparison_flx + n_pix * (1 + (n_pix / n_b)) *
        #                  (gain * F_s + F_dark + read_noise ** 2 + gain ** 2 + sigma_f ** 2)) / gain
        N_comp = [np.sqrt(gain * flx + n_pix * (1 + (n_pix / n_b)) *
                          (gain * F_s + F_dark + read_noise ** 2 + gain ** 2 + sigma_f ** 2)) / gain
                  for flx in comparison_flx]
        N_tar = np.sqrt(gain * target_flx + n_pix * (1 + (n_pix / n_b)) *
                        (gain * F_s + F_dark + read_noise ** 2 + gain ** 2 + sigma_f ** 2)) / gain

        # calculate the total comparison ensemble noise
        N_e_comp = np.sqrt(np.sum(np.array(N_comp) ** 2))

        rel_flux_err = (rel_flx_T1/rel_flux_comps)*np.sqrt((N_tar**2/target_flx**2) +
                                                          (N_e_comp**2/sum(comparison_flx)**2))
        """
        # calculate the total target magnitude and error
        target_magnitude = (-np.log(sum(2.512**-magnitudes_comp))/np.log(2.512)) - \
                           (2.5*np.log10(target_flx/sum(comparison_flx)))

        target_magnitude_error = 2.5*np.log10(1 + np.sqrt(((target_flux_err**2)/(target_flx**2)) +
                                                          (sum(comp_flux_err**2)/sum(comparison_flx)**2)))

        # comparison_magnitude = -(2.5*np.log10(target_flx/sum(comparison_flx)))

        # Append the calculated magnitude and error to the lists
        magnitudes.append(target_magnitude.value[0])
        mag_err.append(target_magnitude_error.value[0])

    # Plot the magnitudes with error bars
    # noinspection PyUnboundLocalVariable
    ax.errorbar(hjd, magnitudes, yerr=mag_err, fmt='o', label="Source_AMag_T1")
    # ax.scatter(hjd, magnitudes, marker='o', color='black')

    # Set the labels and parameters
    fontsize = 14
    ax.set_xlabel('HJD', fontsize=fontsize)
    ax.set_ylabel('Source_AMag_T1', fontsize=fontsize)
    ax.invert_yaxis()
    ax.grid()

    ax.legend(loc="upper right", fontsize=fontsize).set_draggable(True)
    ax.tick_params(axis='both', which='major', labelsize=fontsize)

    plt.show()

    light_curve_data = pd.DataFrame({
        'HJD': hjd,
        'BJD': bjd,
        'Source_AMag_T1': magnitudes,
        'Source_AMag_T1_Error': mag_err
    })

    if not pipeline:
        output_file = input("Enter an output file name and location for the final light curve data in the {} filter "
                            "(ex: C:\\folder1\\folder2\\APASS_254037_B.txt): ".format(filter))
    else:
        output_file = path + "//APASS_254037_" + filter + "_LC_dat.txt"

    light_curve_data.to_csv(output_file, index=False)


def im_plot(image_data, target_aperture, comparison_apertures, target_annulus, comparison_annuli):
    """
    Plot the image with the apertures and annuli overlaid

    Parameters
    ----------
    image_data: array
        Pixel data from the image
    target_aperture: CircularAperture object
        Target aperture location
    comparison_apertures: list of CircularAperture objects
        Comparison aperture locations
    target_annulus: CircularAnnulus object
        Target annulus location
    comparison_annuli: list of CircularAnnulus objects
        Comparison annulus locations

    Returns
    -------
    None
    """
    # First, plot the image
    plt.figure(figsize=(8, 8))
    plt.imshow(image_data, cmap='gray', origin='lower', vmin=np.percentile(image_data, 5),
               vmax=np.percentile(image_data, 95))
    plt.colorbar(label='Counts')

    # Now plot the apertures
    lw = 1.5  # line width
    alpha = 1  # line opacity
    target_aperture.plot(color='darkgreen', lw=lw, alpha=alpha)
    for comparison_aperture in comparison_apertures:
        comparison_aperture.plot(color='red', lw=lw, alpha=alpha)

    # Now plot the annuli
    target_annulus.plot(color='darkgreen', lw=lw, alpha=alpha)
    for comparison_annulus in comparison_annuli:
        comparison_annulus.plot(color='red', lw=lw, alpha=alpha)

    plt.pause(1)
    plt.show()


if __name__ == '__main__':
    main()



import unittest
# from unittest import mock
from unittest.mock import patch
from .IRAF_Reduction import BJD_TDB


class TestBJDTDB(unittest.TestCase):
    def test_BJD_TDB1(self):
        bjd = BJD_TDB("2458403.58763", "lapalma", "00:28:27.97", "78:57:42.66")
        self.assertEqual(bjd.value, 2458403.588447444)
        
    def test_BJD_TDB2(self):
        bjd = BJD_TDB("2458403.58763", "bsuo", "00:28:27.97", "78:57:42.66")
        self.assertEqual(bjd.value, 2458403.588447444)
        
    def test_BJD_TDB3(self):
        bjd = BJD_TDB("2457143.76136", "kpno", "13:27:50.47", "75:55:16.60")
        self.assertEqual(bjd.value, 2457143.762132256)
        
    def test_BJD_TDB4(self):
        bjd = BJD_TDB("2457143.76136", "bsuo", "13:27:50.47", "75:55:16.60")
        self.assertEqual(bjd.value, 2457143.762132256)


unittest.main()



"""
Look up the TESS data and download that data onto a local drive.
Author: Kyle Koeller
Created: 2/19/2022
Last Updated: 04/26/2023
"""

# import required packages
import astroquery.exceptions
from astroquery.mast import Tesscut
from .tesscut import main as tCut
# from tesscut import main as tCut  # testing purposes
from os.path import exists
import pandas as pd

from astropy import units as u
import pkg_resources


def main():
    """
    This function allows the user to enter a TIC ID to be entered, and it also makes sure that number is valid or exists.
    This program will also list off the sector data to be downloaded for cross-referencing if needed.
    :return: Downloaded pixel data in the form of .fits files to be extracted later
    """

    # While loops checks to make sure that the user has entered a valid TIC number that can be found.
    print("\nIf you want to close this program type 'Close' in the following prompt.\n")
    while True:
        try:
            system_name = input("Enter in the TIC-ID given in SIMBAD (TIC 468293391) or the word 'Close' "
                                "to close the program: ")
            sector_table = Tesscut.get_sectors(objectname=system_name)
            break
        except astroquery.exceptions.ResolverError:
            print("\nThe TIC number you entered is invalid or there is no data for this given system.\n")

    if system_name.lower() == "close":
        exit()

    # The ccd info comes from this paper:
    # https://archive.stsci.edu/files/live/sites/mast/files/home/missions-and-data/active-missions/tess/_documents/TESS_Instrument_Handbook_v0.1.pdf
    filename = pkg_resources.resource_filename(__name__, 'tess_ccd_info.txt')
    dc = pd.read_csv(filename, header=None, sep="\t", skiprows=[0])

    gain = dc[3]  # videoscale, gain for the individual camera/ccd
    tess_camera = dc[0]  # camera number
    tess_ccd = dc[1]  # ccd number
    # slice = dc[2]  # could be used in the future

    sector_camera = []
    sector_ccd = []
    for count, val in enumerate(sector_table["camera"]):
        sector_camera.append(val)
        sector_ccd.append(sector_table["ccd"][count])

    # create a tuple form of the sector and TESS camera/ccd data
    list_gain = []
    a = list(zip(tess_camera, tess_ccd))  # tess cam and ccd's
    b = list(zip(sector_camera, sector_ccd))  # sect cam and ccd's

    # compare the TESS and sector information
    for _, sect in enumerate(b):
        for y, tess in enumerate(a):
            if sect == tess:
                list_gain.append(gain[y])

    # splits up list_gain into lists by every 1, 2, 3, or 4th value since there are 4 slices for each camera and ccd
    A = list_gain[::4]
    B = list_gain[1::4]
    C = list_gain[2::4]
    D = list_gain[3::4]

    # append the gain values to the sector table list
    sector_table.add_column(A, name="A gain")
    sector_table.add_column(B, name="B gain")
    sector_table.add_column(C, name="C gain")
    sector_table.add_column(D, name="D gain")

    # sector_table.add_columns(["Gain"])
    # prints off the sector table to let the user know what sectors TESS has observed the object
    while True:
        print("\n The read noise for the cameras is between 7-11 electrons/pixel.")
        print(sector_table)
        print("\nDo you want to download " + "\033[1m" + "\033[93m" + "ALL" + "\033[00m" +
              " the TESS data?")
        download_ans = input("Type 'Yes' to download all Sector data or type 'No' to specify a sector that you want. "
                             "Or type 'Close' to leave: ")
        # prints the word 'ALL' in bold '\033[1m' and in yellow '\033[93m' must return to normal with '\033[0m'
        if download_ans.lower() == "yes":
            print("\nWhen TESS data starts the initial download, it downloads, essentially, a big ZIP file with "
                  "all the individual images inside. Below, please enter the entire file path.")
            print("Example output file path: 'C:\\folder1\\TESS_data\n'")

            for i in sector_table["sector"]:
                download(system_name, i)
                print("\nFinished downloading Sector " + str(i))
        elif download_ans.lower() == "no":
            sector_num = int(input("Which Sector would you like to download: "))
            download(system_name, sector_num)
        elif download_ans.lower() == 'close':
            print("The program will now exit.\n")
            break
        else:
            print("\nPlease enter 'Yes' or 'No' only.\n")

    print("Finished downloading all sector data related to " + system_name + "\n")


def download(system_name, i):
    """
    Download the sector data

    Parameters
    ----------
    system_name - name of the system to download sector data for
    i - sector number

    Returns
    -------
    None
    """
    while True:
        download_path = input("Please enter a file pathway where you want the sector ZIP file to go: ")
        if exists(download_path):
            break
        else:
            print("\nThe file pathway you entered does not exist. Please try again.\n")
    # downloads the pixel file data that can then be analyzed with AIJ
    print("\n\nStarting download of Sector " + str(i))
    manifest = Tesscut.download_cutouts(objectname=system_name, size=[30, 30] * u.arcmin, sector=i,
                                        path=download_path)
    tCut(manifest, download_path)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Calculates the O'Connel Effect based on this paper: https://app.aavso.org/jaavso/article/3511/

Created on Thu Feb 25 00:47:37 2021
Last Edited: 02/27/2023

Original Author: Alec Neal
Last Edits Done By: Kyle Koeller
"""

import matplotlib.pyplot as plt
from .vseq_updated import plot, binning, calc, FT, OConnell
from tqdm import tqdm
import numpy as np
import statistics as st
from os import path

sig_f = lambda f, x, sig_x: abs(f(x + sig_x) - f(x - sig_x)) / 2


def main():
    print("How many filters are you going to use?")
    while True:
        prompt = input("Enter 3 if you are going to use BVR or less than 3 for a combination of them of type 'Close' "
                       "to close the program: ")
        if prompt.isnumeric():
            if 0 < int(prompt) < 4:
                break
            else:
                print("\nYou have entered in a wrong number not between 1-3. Please try again.\n")
        elif prompt.lower() == "close":
            exit()
        else:
            print("\nYou have entered a wrong value. Please try entering a number or the word 'Close'.\n")
    # If statement checks which value was entered for the 'prompt' and corresponds the correct number of files to enter
    # for the O'Connel calculations
    if prompt == "3":
        print("\nPlease enter full file pathways for the following prompt.\n")
        while True:
            infile1 = input("File 1 name: ")
            infile2 = input("File 2 name: ")
            infile3 = input("File 3 name: ")
            if path.exists(infile1) and path.exists(infile2) and path.exists(infile3):
                break
            else:
                print("\nOne of the files you have entered does not exist, please try all three again.\n")
                continue
        hjd = float(input("What is the HJD: "))
        period = float(input("What is the period: "))
        outputile = input("What is the output file name and pathway with .pdf exntension (i.e. C:\\folder1\\test.pdf): ")
        multi_OConnell_total([infile1, infile2, infile3], hjd, period, order=10, sims=1000,
                             sections=4, section_order=7, plot_only=False, save=True, outName=outputile)
    elif prompt == "2":
        print("\nPlease enter full file pathways for the following prompt.\n")
        while True:
            infile1 = input("File 1 name: ")
            infile2 = input("File 2 name: ")
            if path.exists(infile1) and path.exists(infile2):
                break
            else:
                print("\nOne of the files you have entered does not exist, please try all three again.\n")
                continue
        hjd = float(input("What is the HJD: "))
        period = float(input("What is the period: "))
        outputile = input("What is the output file name and pathway with .pdf exntension (i.e. C:\\folder1\\test.pdf): ")
        multi_OConnell_total([infile1, infile2], hjd, period, order=10, sims=1000,
                             sections=4, section_order=7, plot_only=False, save=True, outName=outputile)
    else:
        print("\nPlease enter full file pathways for the following prompt.\n")
        while True:
            infile1 = input("File 1 name: ")
            if path.exists(infile1):
                break
            else:
                print("\nThe file you have entered does not exist, please try all three again.\n")
                continue
        hjd = float(input("What is the HJD: "))
        period = float(input("What is the period: "))
        outputile = input("What is the output file name and pathway with .pdf exntension (i.e. C:\\folder1\\test.pdf): ")
        multi_OConnell_total([infile1], hjd, period, order=10, sims=1000,
                             sections=4, section_order=7, plot_only=False, save=True, outName=outputile)


# print(sig_f(lambda x:1/x,3.4,0.01))
# print(0.01/3.4**2)

def quick_tex(thing):
    plt.rcParams['text.usetex'] = True
    # thing
    plt.rcParams['text.usetex'] = False


dI_phi = lambda b, phase, order: 2 * sum(b[1:order + 1:] * np.sin(2 * np.pi * phase * np.arange(order + 1)[1::]))


def Half_Comp(filter_files, Epoch, period,
              FT_order=10, sections=4, section_order=8,
              resolution=512, offset=0.25, save=False, outName='noname_halfcomp.png',
              title=None, filter_names=None, sans_font=False):
    if sans_font == False:
        plt.rcParams['font.family'] = 'serif'
        plt.rcParams['mathtext.fontset'] = 'dejavuserif'
    bands = len(filter_files)
    axs, fog = plot.multiplot(figsize=(6, 9), dpi=512, height_ratios=[7 / 3 * bands, 3])
    flux = axs[0]
    dI = axs[1]
    colors = ['blue', 'limegreen', 'red', 'm']
    styles = ['--', '-.', ':', '--']

    R2_halves = []
    dI.axhline(0, linestyle='-', color='black', linewidth=1)
    for band in range(bands):
        a, b = binning.polybinner(filter_files[band], Epoch, period, sections=sections,
                                       norm_factor='alt', section_order=section_order)[0][:2:]
        half = int(0.5 * resolution) + 1
        FT1 = FT.FT_plotlist(a, b, FT_order, resolution)
        FT2 = FT.FT_plotlist(a, -1 * b, FT_order, resolution)
        FTphase1, FTflux1 = FT1[0][:half:], FT1[1][:half:]
        FTphase2, FTflux2 = FT2[0][:half:], FT2[1][:half:]

        # R2_halves.append(np.mean([calc.error.CoD(FTflux1,FTflux2),calc.error.CoD(FTflux2,FTflux1)]))
        R2_halves.append(calc.error.CoD(FTflux1, FTflux2))
        R2_halves.append(calc.error.CoD(FTflux2, FTflux1))

        dIlist = []
        for phase in FTphase1:
            dIlist.append(dI_phi(b, phase, FT_order))
        FTflux1 = np.array(FTflux1) + (1 - band) * offset
        FTflux2 = np.array(FTflux2) + (1 - band) * offset
        flux.plot(FTphase1, FTflux1, linestyle=styles[band], color=colors[band])
        flux.plot(FTphase2, FTflux2, '-', color=colors[band])
        # flux.annotate('B',xy=(-0.1,min(FTflux1)))
        if filter_names is not None:
            if len(filter_names) == bands:
                flux.text(-0.12, FTflux1[0], filter_names[band], fontsize=18, rotation=0)
        dI.plot(FTphase1, dIlist, linestyle=styles[band], color=colors[band])

    plt.xlim(-0.025, 0.525)
    if filter_names is None:
        flux.set_ylabel('Flux', fontsize=16)

    # mpl.rcParams['mathtext.fontset'] = 'dejavusans'
    plot.sm_format(flux, numbersize=15, X=0.125, x=0.025, xbottom=False, bottomspine=False, Y=None)
    plot.sm_format(dI, numbersize=15, X=0.125, x=0.025, xtop=False, topspine=False)
    # plt.rcParams['text.usetex'] = True
    # mpl.rcParams['mathtext.fontset'] = 'cm'
    dI.set_xlabel(r'$\Phi$', fontsize=18)
    # plt.rcParams['text.usetex'] = False
    dI.set_ylabel(r'$\Delta I(\Phi)_{\rm FT}$', fontsize=18)
    # dI.set_ylabel(r'$\Delta I(\Phi)_{\rm FT}$',fontsize=14.4)
    if title != '':
        flux.set_title(title, fontsize=14.4, loc='left')
    '''
    print(flux.get_ylim())
    data_int=flux.yaxis.get_data_interval()
    print(data_int)
    allticks=sorted(list(flux.yaxis.get_minorticklocs())+list(flux.yaxis.get_majorticklocs()))
    minorticks=flux.yaxis.get_minorticklocs()
    tick_spacing=min(abs(np.array(minorticks[1::])-np.array(minorticks[:-1:])))
    print(allticks)
    print(tick_spacing)
    flux.set_ylim(top=max(allticks[1:-1])+tick_spacing)#-tick_spacing)
    '''
    # print(flux.yaxis.get_ticks())
    if save:
        plt.savefig(outName, bbox_inches='tight')
        print(outName + ' saved.')
    plt.show()

    # reset to mpl default style
    plt.rcParams['font.family'] = 'sans'
    plt.rcParams['mathtext.fontset'] = 'dejavusans'

    # print('R^2 half =',(calc.error.CoD(FTflux1,FTflux2)+calc.error.CoD(FTflux2,FTflux1))/2)
    # print(R2_halves)
    # print(1-np.mean(R2_halves),'+/-',statistics.stdev(R2_halves))
    # print()

    return print('\nDone.')


def OConnell_total(inputFile, Epoch, period, order, sims=1000,
                   sections=4, section_order=8, norm_factor='alt',
                   starName='', filterName='', FT_order=10, FTres=500):
    """
    This does things.
    Monte Carlo errors are probably underestimates.
    Approximate runtime: ~ sims/1000 minutes.
    glhf
    """

    """
    Generating master parameters from the observational data.
    """
    # ============== DO NOT CHANGE ┬─┬ノ( º _ ºノ) ==============================
    PB = binning.polybinner(inputFile, Epoch, period, sections=sections, norm_factor=norm_factor,
                                 section_order=section_order, FT_order=FT_order)
    c_MB = PB[1][0]
    nc_MB = PB[1][1]
    ob_phaselist = c_MB[0][1]
    ob_fluxlist = c_MB[1][1]
    ob_fluxerr = c_MB[1][2]
    c_sec_phases = c_MB[5][0]
    nc_sec_phases = nc_MB[5][0]
    c_sec_index = c_MB[5][3]
    nc_sec_index = nc_MB[5][3]
    # norm_f = c_MB[4]
    # ob_magerr = c_MB[3][2]
    a = PB[0][0]
    b = PB[0][1]  # Fourier coefficients
    # ==========================================================================

    FTsynth = FT.synth(a, b, ob_phaselist, FT_order)  # synthetic FT curve
    master_simflux = []
    for sim in tqdm(range(sims), desc='Simulating light curves', position=0):
        master_simflux.append(FT.sim_ob_flux(FTsynth, ob_fluxerr))
    # ============

    # ============
    c_master_simflux = []
    nc_master_simflux = []
    master_polyflux = []
    """
    ^List of the resampled polynomial fluxes. Each embedded list is different 
    because the generating data has been Monte Carloed.
    """
    master_FTflux = []  # FT fluxes resulting from the simulations
    master_a = []
    master_b = []  # Lists of the a and b FT coefficients for each sim
    OERlist, LCAlist, dIlist, dIavelist = [], [], [], []

    # = begin sim loop =
    for sim in tqdm(range(sims), desc='Simulation processing', position=0):
        """
        Generating empty lists so that stuff can be inserted into the
        specified [sim] index. Otherwise when trying to append list[sim],
        it would throw an error (there'd be nothing to append to).
        """
        c_master_simflux.append([])
        nc_master_simflux.append([])

        # == begin section loop ===
        for section in range(sections):
            c_master_simflux[sim].append([])  # same as before
            nc_master_simflux[sim].append([])

            """
            'Replacing' the observational fluxes with the Monte Carlo fluxes
            based on the index of each data point when it was in ob_fluxlist
            (the MC fluxes are ordered the same as ob_fluxlist).
            """
            for i in range(len(c_sec_phases[section])):
                c_master_simflux[sim][section].append(master_simflux[sim][c_sec_index[section][i]])
            for i in range(len(nc_sec_phases[section])):
                nc_master_simflux[sim][section].append(master_simflux[sim][nc_sec_index[section][i]])
        # == end section loop == ; resume sim loop

        # Generating polynomial resampled fluxes for each simulation.
        minipoly = binning.minipolybinner(c_sec_phases, c_master_simflux[sim],
                                               nc_sec_phases, nc_master_simflux[sim],
                                               section_order)
        master_polyflux.append(minipoly[1])

        # Calculating various parameters for each simulation.
        FTcoef = FT.coefficients(master_polyflux[sim])
        master_a.append(FTcoef[1])
        master_b.append(FTcoef[2])
        master_FTflux.append(FT.FT_plotlist(master_a[sim], master_b[sim], FT_order, FTres)[1])
        OERlist.append(OConnell.OER_FT(master_a[sim], master_b[sim], FT_order))
        LCAlist.append(OConnell.LCA_FT(master_a[sim], master_b[sim], FT_order, 256))
        dIlist.append(OConnell.Delta_I_fixed(master_b[sim], FT_order))
        dIavelist.append(OConnell.Delta_I_mean_obs_noerror(ob_phaselist, ob_fluxlist, phase_range=0.05))
    # = end sim loop =

    # dIaveerror=st.stdev(dIavelist)#no purpose, compining MC errors with observation errors is double-counting

    # === end Monte Carlo ===

    """
    Calculating FT model errors, as opposed to the errors calculated
    with the simulate light curves.
    """
    a0sig, a0rat = FT.a_sig_fast(a, b, 0, a[0], ob_phaselist, ob_fluxlist, ob_fluxerr, order)
    a_model_err = [a0sig]
    b_model_err = [0]
    a_rat = [a0rat]
    b_rat = [1]
    for n in tqdm(range(1, order + 1), position=0, desc='Calculating FT errors'):
        # TODO, dx0=1 isn't fullproof
        asig, ar = FT.a_sig_fast(a, b, n, a[n], ob_phaselist, ob_fluxlist, ob_fluxerr, order, dx0=1)
        bsig, br = FT.b_sig_fast(a, b, n, b[n], ob_phaselist, ob_fluxlist, ob_fluxerr, order, dx0=1)
        a_model_err.append(asig)
        b_model_err.append(bsig)
        a_rat.append(ar)
        b_rat.append(br)

    # == FT coefficients ==
    a_MC_err = np.array(list(map(st.stdev, zip(*master_a))))
    b_MC_err = np.array(list(map(st.stdev, zip(*master_b))))
    a_total_err = (np.array(a_model_err) ** 2 + a_MC_err[:order + 1:] ** 2) ** 0.5
    b_total_err = (np.array(b_model_err) ** 2 + b_MC_err[:order + 1:] ** 2) ** 0.5

    a2_0125_a2 = a[2] * (0.125 - a[2])
    a2_0125_a2_err = sig_f(lambda x: x * (0.125 - x), a[2], a_total_err[2])

    # == OER ==
    OER = OConnell.OER_FT(a, b, order)
    OER_model_err = OConnell.OER_FT_error_fixed(a, b, a_model_err, b_model_err, order)
    OER_MC_err = st.stdev(OERlist)
    OER_total_err = (OER_model_err ** 2 + OER_MC_err ** 2) ** 0.5

    # == LCA ==
    LCA = OConnell.LCA_FT(a, b, order, 1024)
    LCA_model_err = OConnell.LCA_FT_error(a, b, a_model_err, b_model_err, order, 1024)[1]
    LCA_MC_err = st.stdev(LCAlist)
    LCA_total_err = (LCA_model_err ** 2 + LCA_MC_err ** 2) ** 0.5

    # == Delta_I ==
    Delta_I_025 = OConnell.Delta_I_fixed(b, order)
    Delta_I_025_model_err = OConnell.Delta_I_error_fixed(b_model_err, order)
    Delta_I_025_MC_err = st.stdev(dIlist)
    Delta_I_025_total_err = (Delta_I_025_model_err ** 2 + Delta_I_025_MC_err ** 2) ** 0.5

    # == Delta_I_ave ==
    DIave = OConnell.Delta_I_mean_obs(ob_phaselist, ob_fluxlist, ob_fluxerr, phase_range=0.05, weighted=False)
    Delta_I_ave = DIave[0]
    Delta_I_ave_err = DIave[1]

    # == pizza time ==
    r = lambda x: round(x, 5)

    valerr = lambda x, dx, label, PRECISION=6: print(label + ' =', round(x, PRECISION), '+/-', round(dx, PRECISION))
    # print('\nOER =',r(OER),'+/-',r(OER_total_err))

    print('\n')
    valerr(a[1], a_total_err[1], 'a1')
    valerr(a[2], a_total_err[2], 'a2')
    valerr(a[4], a_total_err[4], 'a4')
    valerr(a[2] * (0.125 - a[2]),
           sig_f(lambda x: x * (0.125 - x), a[2], (a_model_err[2] ** 2 + a_MC_err[2] ** 2) ** 0.5), 'a2(0.125-a2)')
    print('')
    valerr(OER, OER_total_err, 'OER')
    print(r(OER_model_err), r(OER_MC_err), '\n')

    valerr(LCA, LCA_total_err, 'LCA')
    print(r(LCA_model_err), r(LCA_MC_err), '\n')

    valerr(Delta_I_025, Delta_I_025_total_err, 'Delta_I')
    print(r(Delta_I_025_model_err), r(Delta_I_025_MC_err), '\n')

    valerr(Delta_I_ave, Delta_I_ave_err, 'Delta_I_ave')

    # print('LCA =',r(LCA),'+/-',LCA_total_err)
    # print('Delta_I =',Delta_I_025,'+/-',Delta_I_025_total_err)
    # print('')
    return [a, a_total_err], [b, b_total_err], [Delta_I_025, Delta_I_025_total_err], [Delta_I_ave, Delta_I_ave_err], [
        OER, OER_total_err], [LCA, LCA_total_err], [a2_0125_a2, a2_0125_a2_err]


# OConnell_total('NSVS3792718_B_apasscorr.txt',2457288.809054,0.438168,10,1000,sections=2,section_order=8)
# OConnell_total('NSVS3792718_V_apasscorr.txt',2457288.809054,0.438168,10,1000,sections=2,section_order=8)
# OConnell_total('NSVS3792718_R_apasscorr.txt',2457288.809054,0.438168,10,100,sections=2,section_order=8)

# OConnell_total("254037_B.txt", 2458403.58763, 0.317471, 10, 1000, sections=2, section_order=8)

# OConnell_total('NSVS5214334_B3_mags.txt',2459016.769744,0.345551,10,10000,sections=8,section_order=3)
# OConnell_total('NSVS5214334_V4_mags.txt',2459016.769744,0.345551,10,10000,sections=8,section_order=3)
# OConnell_total('NSVS5214334_R3_mags.txt',2459016.769744,0.345551,10,10000,sections=8,section_order=3)

def multi_OConnell_total(filter_files, Epoch, period, order=10,
                         sims=1000, sections=4, section_order=8,
                         norm_factor='alt', starName='', filterNames=[r'$\rm B$', r'$\rm V$', r'$\rm R_C$'],
                         FT_order=10, FTres=500, plot_only=False,
                         plotoff=0.25, save=False, outName='noname.png'):
    """
    does multiple things.
    """

    """
    Half comparison plot. The light curve is mirroed along phase = 0, so that
    similar phases can be compared (should be the in theory). This gives an
    intuitive visual of the O'Connell effect, unlike the barrage of numbers
    this program otherwise spits out.
    """
    Half_Comp(filter_files, Epoch, period, FT_order=order, sections=sections,
              section_order=section_order, offset=plotoff, save=save, outName=outName,
              filter_names=filterNames)

    """
    Actual OConnell stuff, set plot_only=True if you just want the
    half-comparison plot---so you don't have to wait forever just for a plot!
    """
    if plot_only != True:
        filters = len(filter_files)
        a_all = []
        a_err_all = []  # list set up crap
        b_all = []
        b_err_all = []
        dI_FT = []
        dI_FT_err = []
        dI_ave = []
        dI_ave_err = []
        OERs = []
        OERs_err = []
        LCAs = []
        LCAs_err = []
        a22s = []
        a22s_err = []
        """
        Running the O'Connell function on each filter, and placing the resulting
        parameters in corresponding value, value_err lists. E.g. once this completes,
        OERs will contain the OER values for filter 1,2,... and OERs_err contains the errors.
        """
        for band in range(len(filter_files)):
            oc = OConnell_total(filter_files[band], Epoch, period, order, sims=sims,
                                sections=sections, section_order=section_order,
                                norm_factor=norm_factor, FT_order=order, FTres=FTres,
                                filterName='Filter ' + str(band + 1))
            a_all.append(oc[0][0])
            a_err_all.append(oc[0][1])
            b_all.append(oc[1][0])
            b_err_all.append(oc[1][1])
            dI_FT.append(oc[2][0])
            dI_FT_err.append(oc[2][1])
            dI_ave.append(oc[3][0])
            dI_ave_err.append(oc[3][1])
            OERs.append(oc[4][0])
            OERs_err.append(oc[4][1])
            LCAs.append(oc[5][0])
            LCAs_err.append(oc[5][1])
            a22s.append(oc[6][0])
            a22s_err.append(oc[6][1])
            # there's probably an easier way to do this (?), oh well.

        """
        LaTeX table stuff, don't change unless you know what you're doing!
        """
        table_header = '\\begin{table}[H]\n' + '\\begin{center}\n' + '\\begin{tabular}{c|'
        for band in range(filters):
            table_header += 'c'
        table_header += '}\n' + '\\hline\\hline\n' + 'Parameter '
        for band in range(filters):
            table_header += '& Filter ' + str(band + 1)
        table_header += '\\\ \n' + '\\hline\n'

        a1_line = '$a_1$ '
        a2_line = '$a_2$ '
        a4_line = '$a_4$ '
        a22_line = '$a_2(0.125-a_2)$ '
        b1_line = '$2b_1$ '
        dIFT_line = '$\Delta I_{\\rm FT}$ '
        dIave_line = '$\Delta I_{\\rm ave}$ '
        OER_line = 'OER '
        LCA_line = 'LCA '

        strr = lambda x, e=5: str(round(x, e))
        for band in range(filters):
            a1_line += '& $' + strr(a_all[band][1]) + '\pm ' + strr(a_err_all[band][1]) + '$ '
            a2_line += '& $' + strr(a_all[band][2]) + '\pm ' + strr(a_err_all[band][2]) + '$ '
            a4_line += '& $' + strr(a_all[band][4]) + '\pm ' + strr(a_err_all[band][4]) + '$ '
            a22_line += '& $' + strr(a22s[band]) + '\pm ' + strr(a22s_err[band]) + '$ '  # fix
            b1_line += '& $' + strr(2 * b_all[band][1]) + '\pm ' + strr(2 * b_err_all[band][1]) + '$ '
            dIFT_line += '& $' + strr(dI_FT[band]) + '\pm ' + strr(dI_FT_err[band]) + '$ '
            dIave_line += '& $' + strr(dI_ave[band]) + '\pm ' + strr(dI_ave_err[band]) + '$ '
            OER_line += '& $' + strr(OERs[band]) + '\pm ' + strr(OERs_err[band]) + '$ '
            LCA_line += '& $' + strr(LCAs[band]) + '\pm ' + strr(LCAs_err[band]) + '$ '

        lines = [a1_line, a2_line, a4_line, a22_line, b1_line, dIFT_line, dIave_line, OER_line, LCA_line]
        for count, line in enumerate(lines):
            line += '\\\ \n'

        output = table_header
        for count, line in enumerate(lines):
            output += line
        output += '\\hline\n' + '\\end{tabular}\n' + '\\caption{Fourier and O\'Connell stuff (' + str(
            sims) + ' sims)}\n' + '\\label{tbl:OConnell}\n' + '\\end{center}\n' + '\\end{table}\n'
        """
        End LaTeX table stuff.
        """
        print(output)
        outputfile = input("Please enter an output file name without the extension: ")
        file = open(outputfile+".txt", "w")
        file.write(output)
        file.close()
    return 'nada'


# multi_OConnell_total(['NSVS5214334_B3_mags.txt','NSVS5214334_V4_mags.txt','NSVS5214334_R3_mags.txt'],2459016.769744,0.345551,
# order=10,sims=100,sections=8,section_order=3,plot_only=True,plotoff=0.2,save=False,outName='NSVS5214334_BVR_comps.pdf')

# multi_OConnell_total(['NSVS3792718_B_apasscorr.txt','NSVS3792718_V_apasscorr.txt','NSVS3792718_R_apasscorr.txt'],2457288.809054,0.438168,
# order=10,sims=10000,sections=4,section_order=7,plot_only=True)

if __name__ == '__main__':
    main()



"""
Author: Kyle Koeller
Created: 11/11/2020
Last Updated: 07/11/2023

This program is meant to make the process of collecting the different filters from AIJ Excel spreadsheets faster.
The user enters however many nights they have and the program goes through and checks those text files for the
different columns for,HJD, Amag, and Amag error for the B and V filters.

There are error catching statements within the program so if the user mistypes, the program will not crash and
close on them (hopefully)
"""

import pandas as pd
from os import path


def main(c):
    """
    Runs the whole program but collects the number of nights that the user wants to parse through.

    :param c: Number of whether the user has used the program before or not. Must be manually changed within the code
    """
    # warning prompts for the user to read to make sure this program works correctly
    print("\nFrom each night, yous should have a file that is sort of like this: 2018.09.18.APASS.B_datasubset.dat.\n"
          "This file has 7 or 5 columns and you will only need 5 or 3 of them respectively.\n")
    print("All of the nights that you entire MUST have the exact same column number or this program will not work.\n")
    print(
        "You may also type the word 'Close' in the next prompt to leave this program and return to the main menu.\n")
    num = check_num()
    flux_mag(num, c)


def check_num():
    """
    Checks whether the user enters a real number for the number of nights

    :return: returns the entered number
    """
    while True:
        num = input("Number of nights you have: ")
        if num.isnumeric():
            if int(num) > 0:
                break
            else:
                print("You have entered an invalid number. Please try again.\n")
        elif num.lower() == "close":
            exit()
        else:
            print("You have not entered a number or the word 'Close', please try again.\n")
    return int(num)


def file_path(i):
    """
    Checks whether the user enters a real file path

    :param i: the number of the night

    :return: returns the entered file path
    """
    while True:
        file = input("Enter night %d file path: " % (i + 1))
        if path.exists(file):
            break
        else:
            print("You have entered an invalid file path. Please try again.\n")
    df = pd.read_csv(file, delimiter="\t")

    return df


def flux_mag(night, num):
    """
    Finds the flux and magnitude columns from the file

    :param night: the number of the nights
    :param num: which filter is used

    :return:
    """

    total_hjd = []
    total_bjd = []
    total_amag = []
    total_amag_err = []
    total_flux = []
    total_flux_err = []

    for i in range(night):
        df = file_path(i)
        if len(df.columns) == 7:
            # set parameters to lists from the file by the column header
            hjd = []
            bjd = []
            amag = []
            amag_err = []
            flux = []
            flux_err = []
            try:
                if num == 0:
                    hjd = list(df["HJD"])
                elif num == 1:
                    bjd = list(df["BJD_TDB"])
                    hjd = list(df["HJD"])
                amag = list(df["Source_AMag_T1"])
                amag_err = list(df["Source_AMag_Err_T1"])
                flux = list(df["rel_flux_T1"])
                flux_err = list(df["rel_flux_err_T1"])
            except KeyError:
                print("\nThe file you entered does not have the columns of HJD, Source_AMag_T1, or Source_AMag_Err_T1. "
                      "Please re-enter the file path and make sure its the correct file.\n")
                main(0)

            if num == 0:
                total_hjd.append(hjd)
            elif num == 1:
                total_bjd.append(bjd)
                total_hjd.append(hjd)
                new_bjd = [item for elem in total_bjd for item in elem]

            total_amag.append(amag)
            total_amag_err.append(amag_err)
            total_flux.append(flux)
            total_flux_err.append(flux_err)

            # converts the Dataframe embedded lists into a normal flat list
            new_hjd = [item for elem in total_hjd for item in elem]
            new_amag = [item for elem in total_amag for item in elem]
            new_amag_err = [item for elem in total_amag_err for item in elem]
            new_flux = [item for elem in total_flux for item in elem]
            new_flux_err = [item for elem in total_flux_err for item in elem]

            data_amount = 2
        elif len(df.columns == 5):
            # set parameters to lists from the file by the column header
            hjd = []
            amag = []
            amag_err = []
            try:
                if num == 0:
                    hjd = list(df["HJD"])
                elif num == 1:
                    bjd = list(df["BJD_TDB"])
                    hjd = list(df["HJD"])
                amag = list(df["Source_AMag_T1"])
                amag_err = list(df["Source_AMag_Err_T1"])
            except KeyError:
                print("\nThe file you entered does not have the columns of HJD, Source_AMag_T1, or Source_AMag_Err_T1. "
                      "Please re-enter the file path and make sure its the correct file.\n")
                c = 1
                main(c)

            if num == 0:
                total_hjd.append(hjd)
            elif num == 1:
                total_bjd.append(bjd)
                total_hjd.append(hjd)

            total_amag.append(amag)
            total_amag_err.append(amag_err)

            # converts the Dataframe embedded lists into a normal flat list
            new_bjd = [item for elem in total_bjd for item in elem]
            new_hjd = [item for elem in total_hjd for item in elem]
            new_amag = [item for elem in total_amag for item in elem]
            new_amag_err = [item for elem in total_amag_err for item in elem]
            data_amount = 1
        else:
            print("\nThe file you entered does not have the correct amount of columns.\n")
            main(0)

    if data_amount == 1:
        if num == 0:
            data2 = pd.DataFrame({
                "HJD": new_hjd,
                "AMag": new_amag,
                "AMag Error": new_amag_err
            })
        elif num == 1:
            data2 = pd.DataFrame({
                "BJD_TDB": new_bjd,
                "HJD": new_hjd,
                "AMag": new_amag,
                "AMag Error": new_amag_err
            })

        print("")
        output = input("What is the file output name (WITHOUT any file extension): ")

        # output the text files with a designation of magnitude or flux
        data2.to_csv(output + "_magnitudes.txt", index=False, header=False, sep="\t")
        print("")
        print("Fished saving the file to the same location as this program.\n\n")
    elif data_amount == 2:
        # outputs the new file to dataframe and then into a text file for use in Peranso or PHOEBE
        if num == 0:
            data1 = pd.DataFrame({
                "HJD": new_hjd,
                "rel flux": new_flux,
                "rel flux error": new_flux_err
            })

            data2 = pd.DataFrame({
                "HJD": new_hjd,
                "AMag": new_amag,
                "AMag Error": new_amag_err
            })
        elif num == 1:
            data1 = pd.DataFrame({
                "BJD_TDB": new_bjd,
                "HJD": new_hjd,
                "rel flux": new_flux,
                "rel flux error": new_flux_err
            })

            data2 = pd.DataFrame({
                "BJD_TDB": new_bjd,
                "HJD": new_hjd,
                "AMag": new_amag,
                "AMag Error": new_amag_err
            })

        output = input("What is the file output name (WITHOUT any file extension): ")

        # output both text files with a designation of magnitude or flux
        data1.to_csv(output + "_magnitudes.txt", index=False, header=False, sep="\t")
        data2.to_csv(output + "_flux.txt", index=False, header=False, sep="\t")
        print("")


if __name__ == '__main__':
    main(0)



"""
Author: Kyle Koeller
Created: 11/08/2022
Last Edited: 04/19/2024

This program is meant to automatically do the data reduction of the raw images from the
Ball State University Observatory (BSUO) and SARA data. The new calibrated images are placed into a new folder as to
not overwrite the original images.
"""
# import sys
from pathlib import Path
import time
import warnings

from astropy import wcs
from astropy.stats import mad_std
from astropy import units as u
from astropy.nddata import CCDData
from astropy.io import fits
from astropy.time import Time
from astropy.coordinates import SkyCoord, EarthLocation

import ccdproc as ccdp
import numpy as np
import matplotlib.pyplot as plt

# turn off this warning that just tells the user,
# "The warning raised when the contents of the FITS header have been modified to be standards compliant."
warnings.filterwarnings("ignore", category=wcs.FITSFixedWarning)

# global variables for combining, to either play with or set depending on the run
sigma_clip_low_thresh = None
sigma_clip_high_thresh = 3
sigclip = 5  # sigclip for cosmic ray removal
# rdnoise = 10.83  # * u.electron  # gathered from fits headers manually
# gain = 1.43  # * u.electron / u.adu  # gathered from fits headers manually
overwrite = True  # if the user wants to overwrite already existing files or not, by default it is set to True
# mem_limit = 1600e6  # maximum memory limit 4.5 Gb is the recommended which is 450e6 (i.e. 8.0 Gb would be 800e6)
dark_bool = "True"
# location = "bsuo"


def main(path="", calibrated="", pipeline=False, location="", dark_bool=True, gain=1.43, rdnoise=10.83, mem_limit=450e6):
    """
    This function calls all other functions in order of the calibration.

    :param path: the path to the raw images
    :param calibrated: the path to the calibrated images
    :param pipeline: if the user wants to use the pipeline or not
    :param location: the location of the telescope
    :param dark_bool: if the user wants to use dark frames or not

    :return: outputs all calibration images into a new reduced folder designated by the user.
    """
    if not pipeline:
        # allows the user to input where the raw images are and where the calibrated images go to
        path = input("Please enter a file pathway (i.e. C:\\folder1\\folder2\\[raw]) to where the raw images are or type "
                     "the word 'Close' to leave: ")
        # path = "C:\\Users\\Kyle\\OneDrive\\PhysicsAstro\\Astronomy\\Code\\IRAF\\Calibration"
        if path.lower() == "close":
            exit()
        # path = "Calibration2"
        calibrated = input("Please enter a file pathway for a new calibrated folder to not overwrite the original images "
                           "(C:\\folder1\\folder2\\[calibrated]): ")
        # calibrated = "C:\\test"
        # checks whether the file paths from above are real
        while True:
            try:
                images_path = Path(path)
                calibrated_data = Path(calibrated)
                break
            except FileNotFoundError:
                print("Files were not found. Please try again.\n")
                path = input("Please enter a file path or folder name (if this code is in the same main folder): ")
                calibrated = input("Please enter a name for a new calibrated folder to not overwrite the original images: ")

        if location.lower() == "kpno":
            gain, rdnoise = kpno()
        elif location.lower() == "ctio":
            gain, rdnoise = ctio()
        elif location.lower() == "lapalma":
            gain, rdnoise = lapalma()
        elif location.lower() == "bsuo":
            pass
        else:
            print("\nDo you want to load default options like gain and read noise? The defaults are for BSUO")
            while True:
                default_ans = input("To load defaults type 'Default' otherwise type 'New' to enter values: ")
                # default_ans = "default"
                if default_ans.lower() == "default":
                    break
                elif default_ans.lower() == "new":
                    new_default()
                    break
                else:
                    print("Please either enter 'Default' or 'New'.\n")

        calibrated_data.mkdir(exist_ok=True)
        files = ccdp.ImageFileCollection(images_path)

        zero, overscan_region, trim_region = bias(files, calibrated_data, path, pipeline, mem_limit, gain)
        if not dark_bool:
            master_dark = None
        else:
            master_dark = dark(files, zero, calibrated_data, overscan_region, trim_region, gain, rdnoise, mem_limit)

        flat(files, zero, master_dark, calibrated_data, overscan_region, trim_region, gain, rdnoise, mem_limit)
        science_images(files, calibrated_data, zero, master_dark, trim_region, overscan_region)
    else:
        # checks whether the file paths from above are real
        while True:
            try:
                images_path = Path(path)
                calibrated_data = Path(calibrated)
                break
            except FileNotFoundError:
                print("Files were not found. Please try again.\n")
                path = input("Please enter a file path or folder name (if this code is in the same main folder): ")
                calibrated = input(
                    "Please enter a name for a new calibrated folder to not overwrite the original images: ")

        if location.lower() == "kpno":
            kpno()
        elif location.lower() == "ctio":
            ctio()
        elif location.lower() == "lapalma":
            lapalma()
        elif location.lower() == "bsuo":
            pass

        calibrated_data.mkdir(exist_ok=True)
        files = ccdp.ImageFileCollection(images_path)

        zero, overscan_region, trim_region = bias(files, calibrated_data, path, pipeline, mem_limit, gain)
        if not dark_bool:
            master_dark = None
        else:
            master_dark = dark(files, zero, calibrated_data, overscan_region, trim_region, gain, rdnoise, mem_limit)

        flat(files, zero, master_dark, calibrated_data, overscan_region, trim_region, gain, rdnoise, mem_limit)
        science_images(files, calibrated_data, zero, master_dark, trim_region, overscan_region)

# more observatory sites can be found here: https://github.com/astropy/astropy-data/blob/gh-pages/coordinates/sites.json
def kpno():
    """
    Kitt Peak National Observatory default values

    :return: Camera characteristics
    """
    global dark_bool
    
    gain = 2.3
    rdnoise = 6.0
    dark_bool = True

    return gain, rdnoise


def ctio():
    """
    Cerro Tololo Inter-American Observatory default values

    :return: Camera characteristics
    """
    global dark_bool

    gain = 2.0
    rdnoise = 9.7
    dark_bool = True

    return gain, rdnoise


def lapalma():
    """
    La Palma default values

    :return: Camera characteristics
    """
    global dark_bool

    gain = 1.0
    rdnoise = 6.3
    dark_bool = True

    return gain, rdnoise


def new_default():
    """
    Generates new values that the user can enter

    :return: newly entered default values
    """
    global sigma_clip_high_thresh
    global sigma_clip_low_thresh
    global sigclip
    global dark_bool
    global location

    sigma_clip_low_thresh = (input("\nEnter a sigma clip low threshold, default is 'None': "))
    if sigma_clip_low_thresh.lower() == "none":
        sigma_clip_low_thresh = None
    else:
        sigma_clip_low_thresh = int(sigma_clip_high_thresh)
    sigma_clip_high_thresh = int(input("Enter a sigma clip high threshold, default is '3': "))
    gain = float(input("Enter a gain value, default is '1.43' electron/adu: "))
    rdnoise = float(input("Enter a readnoise value, default is '10.83' electrons: "))
    sigclip = int(input("Enter a sigma clip value for cosmic ray removal, default is '5': "))
    dark_bool = input("Are you using Dark Frames, default is True (enter 'True' or 'False'): ")
    if dark_bool.lower == "false":
        dark_bool = False
    elif dark_bool.lower == "true":
        dark_bool = True
    location = input("What is your observation location, default is bsuo (ctio, kpno, lapalma")

    return gain, rdnoise, dark_bool


def reduce(ccd, overscan_region, trim_region, num, zero, combined_dark, good_flat, gain, rdnoise):
    """
    This function takes the information for each section of the reduction process into a singular function for
    limits in duplication of the code.

    :param ccd: individual image
    :param overscan_region: the overscan region of the image
    :param trim_region: region of the image to trim
    :param num: tells the program what stage of the process it is in
    :param zero: master bias
    :param combined_dark: master dark
    :param good_flat: master flat
    :param gain: gain of the camera
    :param rdnoise: readout noise of the camera

    :return: depends on the stage of process, but will return a final image for each process
    """

    # subtract overscan, trims image, and gain corrects
    if overscan_region.lower() == "none":
        pass
    else:
        # new_ccd = ccdp.ccd_process(ccd, fits_section=overscan_region, trim=trim_region, gain=gain * u.electron / u.adu,
        #                            readnoise=rdnoise * u.electron, error=False)
        ccd = ccdp.subtract_overscan(ccd, fits_section=overscan_region, median=True, overscan_axis=None)

    trim_ccd = ccdp.trim_image(ccd, fits_section=trim_region)
    new_ccd = ccdp.gain_correct(trim_ccd, gain=gain * u.electron / u.adu)  # gain correct the image

    # this if statement checks whether the input is bias, dark, flat, or science reduction
    # bias combining
    if num == 0:
        return new_ccd
    # dark combining
    elif num == 1:
        # print(new_ccd.unit, zero.unit)
        # Subtract bias
        sub_ccd = ccdp.subtract_bias(new_ccd, zero)

        return sub_ccd
    # flat combining
    elif num == 2:
        # print(new_ccd.unit, zero.unit)
        # Subtract bias
        sub_ccd = ccdp.subtract_bias(new_ccd, zero)

        # Subtract the dark current
        if not dark_bool:
            final_ccd = sub_ccd
        else:
            final_ccd = ccdp.subtract_dark(sub_ccd, combined_dark, exposure_time='exptime', exposure_unit=u.second,
                                           scale=True)
        return final_ccd
    # science calibration
    elif num == 3:
        # Subtract bias
        if location.lower() == "bsuo":
            sub_ccd = ccdp.subtract_bias(new_ccd, zero)
        else:
            sub_ccd = ccdp.subtract_bias(new_ccd, zero)
        # Subtract the dark current
        if not dark_bool:
            reduced = sub_ccd
        else:
            reduced = ccdp.subtract_dark(sub_ccd, combined_dark, exposure_time='exptime', exposure_unit=u.second,
                                         scale=True)
        # flat field correct the science image based on filter
        reduced = ccdp.flat_correct(ccd=reduced, flat=good_flat, min_value=1.0)

        # cosmic ray reject above 5 sigmas and gain_apply is set to false because it changes the units of the image
        # new_reduced = ccdp.cosmicray_lacosmic(reduced, gain_apply=False, readnoise=rdnoise, gain=gain, sigclip=sigclip)

        return reduced


def bias(files, calibrated_data, path, pipeline, mem_limit, gain):
    """
    Calibrates the bias images

    :param path: the raw images folder path
    :param files: file location where all raw images are
    :param calibrated_data: file location where the new images go
    :param pipeline: true or false for pipeline usage
    :param mem_limit: maximum memory chunk usage
    :param gain: gain of the camera

    :return: the combined bias image and the trim and overscan regions
    """

    # plots one of the flat image mean count values across all columns to find the trim and overscan regions
    if not pipeline:
        print("\n\nThe flat image that you enter next should be inside the " + "\033[1m" + "\033[93m" + "FIRST" +
              "\033[00m" + " folder that you entered above or this will crash.")
        while True:
            try:
                image = input(
                    "Please enter the name of one of the flat image to be looked at for overscan and data regions: ")
                # image = "Flat-Empty-B-Bin2-001-NoGEM.fts"  # testing
                cryo_path = Path(path)
                bias_1 = CCDData.read(cryo_path / image, unit='adu')
                break
            except FileNotFoundError:
                print("\nThe file you entered could not be found, please try entering " 
                      "\033[1m" + "\033[93m" + "JUST" + "\033[00m" + " the file name only.\n")
        # bias_1 = CCDData.read(cryo_path / 'bias-0001.fits', unit='adu')  # testing

        print("\n\nFor the overscan region, [columns, rows], and if you want all the columns then you want would enter, \n"
              "[1234:5678, 1234:5678] and this would say rows between those values and all the columns. \n"
              "This would also work if you wanted all the columns ([: , 1234:5678]).\n")
        bias_plot(bias_1)

        overscan_region = input("Please enter the overscan region you determined from the figure.\n"
                                "Example '[2073:2115, :]' or if you do not have an overscan region enter 'None': ")
        trim_region = input("Please enter the data section. Example '[20:2060, 12:2057]': ")
        print()
    else:
        overscan_region = "[2073:2115, :]"
        trim_region = "[20:2060, 12:2057]"

    print("\nStarting overscan on bias.\n")
    for ccd, file_name in files.ccds(imagetyp='BIAS', return_fname=True, ccd_kwargs={'unit': 'adu'}):
        new_ccd = reduce(ccd, overscan_region, trim_region, 0, zero=None, combined_dark=None, good_flat=None, gain=gain, rdnoise=rdnoise)

        list_of_words = file_name.split(".")
        new_fname = "{}.fits".format(list_of_words[0])

        # Save the result
        new_ccd.write(calibrated_data / new_fname, overwrite=overwrite)

        add_header(calibrated_data, new_fname, "BIAS", "None", None, None, None,
                   trim_region, overscan_region, gain, rdnoise)

        # output that an image is finished for updates to the user
        print("Finished overscan correction for " + str(new_fname))

    print("\nFinished overscan correcting bias frames.")
    # combine all the output bias images into a master bias
    reduced_images = ccdp.ImageFileCollection(calibrated_data)
    calibrated_biases = reduced_images.files_filtered(imagetyp='BIAS', include_path=True)

    combined_bias = ccdp.combine(calibrated_biases,
                                 method='average',
                                 sigma_clip=True, sigma_clip_low_thresh=sigma_clip_low_thresh,
                                 sigma_clip_high_thresh=sigma_clip_high_thresh,
                                 sigma_clip_func=np.ma.median, signma_clip_dev_func=mad_std,
                                 mem_limit=mem_limit
                                 )

    fname = "zero.fits"
    combined_bias.meta['combined'] = True
    combined_bias.write(calibrated_data / fname, overwrite=overwrite)

    add_header(calibrated_data, fname, "BIAS", "None", None, None, None,
               trim_region, overscan_region, gain, rdnoise)

    print("\nFinished creating zero.fits\n")

    return combined_bias, overscan_region, trim_region


def bias_plot(ccd):
    """
    Plots the count values for row 1000 to find the overscan and trim regions

    :param ccd: bias image to be looked at
    :return: None
    """
    plt.figure(figsize=(10, 5))
    plt.plot(ccd.data[1000][:], label='Raw Flat')
    plt.grid()
    plt.axvline(x=2077, color='black', linewidth=2, linestyle='dashed', label='Suggested Start of Overscan')
    plt.legend()
    # plt.ylim(0, 60000)
    plt.xlim(-50, 2130)
    plt.xlabel('pixel number')
    plt.ylabel('Counts')
    # plt.title('Count Values for Row 1000')
    # plt.show()


def dark(files, zero, calibrated_path, overscan_region, trim_region, gain, rdnoise, mem_limit):
    """
    Calibrates the dark frames.

    :param files: file location of raw images
    :param zero: master bias image
    :param calibrated_path: file location for the new images
    :param trim_region: trim region for images
    :param overscan_region: overscan region for images
    :param mem_limit: maximum memory chunk usage
    :param gain: gain of the camera
    :param rdnoise: readout noise of the camera

    :return: combined master dark
    """
    print("Starting dark calibration.\n")
    # calibrating a combining the dark frames
    for ccd, file_name in files.ccds(imagetyp='DARK', ccd_kwargs={'unit': 'adu'}, return_fname=True):
        sub_ccd = reduce(ccd, overscan_region, trim_region, 1, zero, combined_dark=None, good_flat=None, gain=gain, rdnoise=rdnoise)

        # new file name that uses the number from the original image
        list_of_words = file_name.split(".")
        new_fname = "{}.fits".format(list_of_words[0])

        # Save the result
        sub_ccd.write(calibrated_path / new_fname, overwrite=overwrite)

        add_header(calibrated_path, new_fname, "DARK", "None", None, None, None,
                   trim_region, overscan_region, gain, rdnoise)

        print("Finished overscan correction and bias subtraction for " + str(new_fname))

    print("\nFinished overscan correcting and bias subtracting all dark frames.")
    print("\nStarting combining dark frames.\n")
    time.sleep(10)
    reduced_images = ccdp.ImageFileCollection(calibrated_path)
    calibrated_darks = reduced_images.files_filtered(imagetyp='dark', include_path=True)

    combined_dark = ccdp.combine(calibrated_darks,
                                 method='average',
                                 sigma_clip=True, sigma_clip_low_thresh=sigma_clip_low_thresh,
                                 sigma_clip_high_thresh=sigma_clip_high_thresh,
                                 sigma_clip_func=np.ma.median, signma_clip_dev_func=mad_std,
                                 rdnoise=rdnoise * u.electron, gain=gain * u.electron / u.adu, mem_limit=mem_limit
                                 )

    fname = "master_dark.fits"
    combined_dark.meta['combined'] = True
    combined_dark.write(calibrated_path / fname, overwrite=overwrite)

    add_header(calibrated_path, fname, "DARK", "None", None, None, None,
               trim_region, overscan_region, gain, rdnoise)

    print("Finished creating a master dark.\n")
    return combined_dark


def flat(files, zero, combined_dark, calibrated_path, overscan_region, trim_region, gain, rdnoise, mem_limit):
    """
    Calibrate flat images.

    :param files: file location for raw images
    :param zero: combined bias image
    :param combined_dark: combined bias image
    :param calibrated_path: file location for new images
    :param trim_region: trim region for images
    :param overscan_region: overscan region for images
    :param mem_limit: maximum memory chunk usage
    :param gain: gain of the camera
    :param rdnoise: readout noise of the camera

    :return: master flat files in each filter
    """
    print("Starting flat calibration.\n")

    # calibrating and combining the flat frames
    for ccd, file_name in files.ccds(imagetyp='FLAT', return_fname=True, ccd_kwargs={'unit': 'adu'}):
        final_ccd = reduce(ccd, overscan_region, trim_region, 2, zero, combined_dark, good_flat=None, gain=gain, rdnoise=rdnoise)

        # new file name with the filter and number from the original file
        list_of_words = file_name.split(".")
        new_fname = "{}.fits".format(list_of_words[0])

        # Save the result
        final_ccd.write(calibrated_path / new_fname, overwrite=overwrite)

        add_header(calibrated_path, new_fname, "FLAT", "None", None, None, None,
                   trim_region, overscan_region, gain, rdnoise)

        print("Finished overscan correction, bias subtraction, and dark subtraction for " + str(new_fname))

    print("\nFinished overscan, bias subtracting, and dark subtracting of flat frames.\n")
    print("Starting flat combination.\n")
    time.sleep(10)

    ifc = ccdp.ImageFileCollection(calibrated_path)
    flat_filters = set(h['FILTER'] for h in ifc.headers(imagetyp="FLAT"))
    for filt in flat_filters:
        to_combine = ifc.files_filtered(imagetyp="flat", filter=filt, include_path=True)
        combined_flats = ccdp.combine(to_combine,
                                      method='median',
                                      sigma_clip_high_thresh=sigma_clip_high_thresh,
                                      sigma_clip_func=np.ma.median, signma_clip_dev_func=mad_std,
                                      rdnoise=rdnoise * u.electron, gain=gain * u.electron / u.adu,
                                      mem_limit=mem_limit
                                      )

        combined_flats.meta['combined'] = True
        flat_file_name = 'master_flat_{}.fits'.format(filt.replace("Empty/", ""))

        combined_flats.write(calibrated_path / flat_file_name, overwrite=overwrite)

        add_header(calibrated_path, flat_file_name, "FLAT", "None", None, None, None,
                   trim_region, overscan_region, gain, rdnoise)

        print("Finished combining flat " + str(flat_file_name))

    print("\nFinished creating the master flats by filter.\n")


def science_images(files, calibrated_data, zero, combined_dark, trim_region, overscan_region):
    all_reds = []
    science_imagetyp = 'LIGHT'
    flat_imagetyp = 'FLAT'

    ifc_reduced = ccdp.ImageFileCollection(calibrated_data)
    combined_flats = {ccd.header['filter']: ccd for ccd in ifc_reduced.ccds(imagetyp=flat_imagetyp, combined=True)}

    print("Starting reduction of science images.\n")

    for light, file_name in files.ccds(imagetyp=science_imagetyp, return_fname=True, ccd_kwargs={'unit': 'adu'}):
        good_flat = combined_flats[light.header['filter']]
        reduced = reduce(light, overscan_region, trim_region, 3, zero, combined_dark, good_flat, gain=gain, rdnoise=rdnoise)

        list_of_words = file_name.split(".")
        new_fname = "{}.fits".format(list_of_words[0])

        all_reds.append(reduced)
        reduced.write(calibrated_data / new_fname, overwrite=overwrite)

        hjd = light.header["JD-HELIO"]
        ra = light.header["RA"]
        dec = light.header["DEC"]

        add_header(calibrated_data, new_fname, science_imagetyp, "None", hjd, ra, dec, trim_region, overscan_region, gain, rdnoise)

        print("Finished calibration of " + str(new_fname))
    print("\nFinished calibrating all science images.")


def add_header(pathway, fname, imagetyp, filter_name, hjd, ra, dec, trim_region, overscan_region, gain, rdnoise):
    """
    Adds values to the header of each image reduced

    :param overscan_region: BIASSEC for the header
    :param trim_region: DATASEC for the header
    :param dec: declination of target object
    :param ra: right ascension of target object
    :param hjd: time of mid-exposure for each image
    :param pathway: pathway to the new file
    :param fname: file name
    :param imagetyp: image type (bias, flat, dark, light)
    :param filter_name: filter type (B, V, R)
    :param gain: gain of the camera
    :param rdnoise: readout noise of the camera

    :return: None
    """
    # bias_image = get_pkg_data_filename(pathway + "\\" + fname)
    image_name = pathway / fname
    fits.setval(image_name, "GAIN", value=gain, comment="Units of e-/ADU")
    fits.setval(image_name, "RDNOISE", value=rdnoise, comment="UNits of e-")
    fits.setval(image_name, "OBSERVAT", value=location, comment="Obs. Loc.")
    fits.setval(image_name, "IMAGETYP", value=imagetyp, comment="Image type")
    fits.setval(image_name, "DATASEC", value=trim_region, comment="Trim data section")
    fits.setval(image_name, "BIASSEC", value=overscan_region, comment="Overscan section")
    fits.setval(image_name, "EPOCH", value="J2000.0")
    # fits.setval(bias_image, "FILTER", value=filter_name)
    if imagetyp == "LIGHT":
        bjd = BJD_TDB(hjd, location, ra, dec)
        fits.setval(image_name, "BJD_TDB", value=bjd.value, comment="Bary. Julian Date, Bary. Dynamical Time")


def BJD_TDB(hjd, site_name, ra, dec):
    """
    Converts HJD to BJD_TDB

    :param ra: right ascension of target object
    :param dec: declination of target object
    :param hjd: HJD time for the mid-exposure image
    :param loc: location of the observations
    :return: Newly calculated BJD_TDB that is light time corrected
    """
    
    if site_name == "bsuo":
        # set the latitude and longitude of the BSUO site manually
        loc = {
            'lon': -85.411896,  # degrees
            'lat': 40.199879,  # degrees
            'elevation': 0.2873  # kilometers
        }
        obs = EarthLocation.from_geodetic(loc["lat"], loc["lon"], loc["elevation"])
    else:
        loc = EarthLocation.of_site(site_name)
        # need to gather the specific lat, long, and elev. from the .geodetic command. This specifically lists information on each of the parameters but we only want the value and not the units
        obs = EarthLocation.from_geodetic(loc.geodetic[0].value, loc.geodetic[1].value, loc.geodetic[2].value/1000) # converts the meters of the elevation to km
        
    helio = Time(hjd, scale='utc', format='jd')
    star = SkyCoord(ra, dec, unit=(u.hour, u.deg))
    ltt = helio.light_travel_time(star, 'heliocentric', location=obs)
    guess = helio - ltt
    # if we assume guess is correct - how far is heliocentric time away from true value?
    delta = (guess + guess.light_travel_time(star, 'heliocentric', obs)).jd - helio.jd
    # apply this correction
    guess -= delta * u.d

    ltt = guess.light_travel_time(star, 'barycentric', obs)
    return guess.tdb + ltt


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Author: John Kielkopf (University of Louisville)
Created: Unknown

Editor: Kyle Koeller
Last Edited: 05/07/2023

Paper is: https://ui.adsabs.harvard.edu/abs/2019ascl.soft05007B/abstract
"""

# !/usr/local/bin/python3

import numpy as np
import astropy.io.fits as pyfits
from time import gmtime, strftime  # for utc
from .vseq_updated import conversion
# from vseq_updated import conversion  # testing purposes

from astropy.time import Time
from astropy.coordinates import EarthLocation, SkyCoord
from astropy import units as u

"""
Extract all images from a TESS pixel BINTABLE file
Removes extended headers
Selects only the image data
Detects and does not convert low quality
"""


def main(search_file, pathway):
    # print(str(search_file))
    infile = "tess" + str(search_file).split("tess")[1]  # gets the actual sector file
    # print(infile)
    # pathway = str(search_file).split("tess")[0]  # gets the file pathway
    print("\nThe program will use the file pathway that you entered previously and will now ask for a prefix "
          "to each file name.")

    print("Example prefix might be 'NSVS_896797_[program adds stuff here]'\n")
    outprefix = input("Please enter what you want each file to always have in its name: ")

    # Set an overwrite flag True so that images can be overwritten
    # Otherwise set it False for safety

    overwriteflag = True

    # Open the fits file readonly by default and create an input hdulist
    inlist = pyfits.open(pathway + "\\" + infile)
    # Assign the input headers

    # Master

    # inhdr0 = inlist[0].header

    # Sector information including range of dates
    # For more information inspect the entries in inhdr1
    # Is there a DQUALITY flag?

    # inhdr1 = inlist[1].header

    # Coordinates

    inhdr2 = inlist[2].header

    # Create a new fits header for the pixel file images

    newhdr = inhdr2

    # Clear the instrument and telescope keywords
    # AIJ may use them in development software that could break processing

    del newhdr['INSTRUME']
    del newhdr['TELESCOP']
    del newhdr['CHECKSUM']

    # Clear the extension name which will not apply to the cutout slices

    del newhdr['EXTNAME']

    imagedata = inlist[1].data
    nimages = np.size(imagedata)

    # Diagnostics

    # print(np.size(imagedata))
    # print(len(imagedata[0]))

    print("Starting to check images. This may take a few minutes. \n")
    for i in range(nimages):
        # Get image data

        inimage = imagedata[i][4]

        # Get BJD -2457000

        bjd0 = 2457000.
        bjd1 = imagedata[i][0]
        quality_flag = imagedata[i][8]
        tess_ffi = imagedata[i][11]

        if np.isnan(bjd1):
            print('Image ', i + 1, 'skipped: lacks valid BJD timestamp.')

        elif quality_flag != 0:
            print('Image ', i + 1, 'skipped: poor quality.')

        else:
            # calculate the mid-exposure in BJD
            bjd = bjd0 + bjd1
            outimage = inimage

            split = infile.split("_")
            ra = conversion([str(float(split[1]) / 15)])[0]
            dec = conversion([split[2]])[0]

            hjd = bary_to_helio(ra, dec, bjd, "greenwich")

            outlist = pyfits.PrimaryHDU(outimage.astype('float32'), newhdr)

            # Provide a new date stamp

            file_time = strftime("%Y-%m-%d %H:%M:%S", gmtime())

            # Update the header
            # Append data to the header from the other header blocks of the bintable file

            outhdr = outlist.header
            outhdr['LST_UPDT'] = file_time
            outhdr['BJD_TDB'] = bjd
            outhdr['HJD'] = hjd.value
            outhdr['COMMENT'] = tess_ffi
            # outhdr['history'] = 'Image from ' + infile

            # Write the fits file

            outfile = pathway + "/" + outprefix + 'tess_%05d.fits' % (i,)
            # print(outfile)
            outlist.writeto(r"" + outfile, overwrite=overwriteflag)

    print("Finished checking all images.\n")
    # Close the input  and exit
    inlist.close()


def bary_to_helio(ra, dec, bjd, obs_name):
    bary = Time(bjd, scale='tdb', format='jd')
    obs = EarthLocation.of_site(obs_name)
    star = SkyCoord(ra, dec, unit=(u.hour, u.deg))
    ltt = bary.light_travel_time(star, 'barycentric', location=obs)
    guess = bary - ltt
    delta = (guess + guess.light_travel_time(star, 'barycentric', obs)).jd - bary.jd
    guess -= delta * u.d

    ltt = guess.light_travel_time(star, 'heliocentric', obs)
    return guess.utc + ltt


# main("tess-s0059-3-1_7.116535_78.961849_88x88_astrocut.fits", "C:\\New folder")
